import { OnDestroy, OnInit } from '@angular/core';
import { ILatLong } from '../interfaces/ilatlong';
import { MapService } from '../services/map.service';
/**
 * GeoTracker renders the current mouse geo-coordinates on the map canvas.
 *
 * ### Example
 * ```typescript
 * import {Component} from '@angular/core';
 * import {MapComponent, MapGeoTracker} from '...';
 *
 * @Component({
 *  selector: 'my-map-cmp',
 *  styles: [`
 *    .map-container { height: 300px; }
 * `],
 *  template: `
 *    <x-map [Latitude]="lat" [Longitude]="lng" [Zoom]="zoom">
 *      <x-geo-tracker [Attachment]="topLeft" [xOffset]="10" [yOffset]="20"></x-geo-tracker>
 *     </x-map>
 *  `
 * })
 * ```
 *
 * @export
 * @class InfoBoxComponent
 * @implements {OnInit}
 * @implements {OnDestroy}
 */
export declare class MapGeoTrackerComponent implements OnDestroy, OnInit {
    private _mapService;
    private _position;
    private _subscription;
    /**
     * The horizontal offset of the GeoTracker from the attachment point.
     *
     * @type {number}
     * @memberof MapGeoTrackerComponent
     */
    xOffset: number;
    /**
     * The vertical offset of the GeoTracker from the attachement point.
     *
     * @type {number}
     * @memberof MapGeoTrackerComponent
     */
    yOffset: number;
    /**
     * The attachment of the Geo Tracker relative to the map canvas.
     *
     * @type {Attachment}
     * @memberof MapGeoTrackerComponent
     */
    Attachment: Attachment;
    /**
     * Gets the geo coordinates of the current cursor position on the map.
     *
     * @readonly
     * @type {ILatLong}
     * @memberof MapGeoTrackerComponent
     */
    readonly Position: ILatLong;
    /**
     * Creates an instance of InfoBoxComponent.
     * @param {MapService} _mapService - Concrete {@link MapService} implementation for underlying Map architecture.
     *
     * @memberof MapGeoTrackerComponent
     */
    constructor(_mapService: MapService);
    /**
     * Called on after component view as been initialized. Part of the ng Component life cycle.
     *
     * @returns {void}
     *
     * @memberof MapGeoTrackerComponent
     */
    ngOnInit(): void;
    /**
     * Called on component destruction. Frees the resources used by the component. Part of the ng Component life cycle.
     *
     * @returns {void}
     *
     * @memberof MMapGeoTrackerComponentap
     */
    ngOnDestroy(): void;
    /**
     * Updates the tracker coordinates.
     *
     * @private
     * @param {ILatLong} position - The position to update.
     * @memberof MapComponent
     */
    private Track(position);
}
/**
 * Enumeration defining the attachment position of the tracker on the map.
 */
export declare enum Attachment {
    TopLeft = 0,
    TopCenter = 1,
    TopRight = 2,
    MiddleLeft = 3,
    MiddleRight = 4,
    BottomLeft = 5,
    BottomCenter = 6,
    BottomRight = 7,
}
