"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var map_service_1 = require("../services/map.service");
/**
 * GeoTracker renders the current mouse geo-coordinates on the map canvas.
 *
 * ### Example
 * ```typescript
 * import {Component} from '@angular/core';
 * import {MapComponent, MapGeoTracker} from '...';
 *
 * @Component({
 *  selector: 'my-map-cmp',
 *  styles: [`
 *    .map-container { height: 300px; }
 * `],
 *  template: `
 *    <x-map [Latitude]="lat" [Longitude]="lng" [Zoom]="zoom">
 *      <x-geo-tracker [Attachment]="topLeft" [xOffset]="10" [yOffset]="20"></x-geo-tracker>
 *     </x-map>
 *  `
 * })
 * ```
 *
 * @export
 * @class InfoBoxComponent
 * @implements {OnInit}
 * @implements {OnDestroy}
 */
var MapGeoTrackerComponent = /** @class */ (function () {
    ///
    /// Constructor
    ///
    /**
     * Creates an instance of InfoBoxComponent.
     * @param {MapService} _mapService - Concrete {@link MapService} implementation for underlying Map architecture.
     *
     * @memberof MapGeoTrackerComponent
     */
    function MapGeoTrackerComponent(_mapService) {
        this._mapService = _mapService;
        ///
        /// Field declarations
        ///
        this._position = null;
        this._subscription = null;
    }
    Object.defineProperty(MapGeoTrackerComponent.prototype, "Position", {
        ///
        /// Property declarations.
        ///
        /**
         * Gets the geo coordinates of the current cursor position on the map.
         *
         * @readonly
         * @type {ILatLong}
         * @memberof MapGeoTrackerComponent
         */
        get: function () {
            return this._position;
        },
        enumerable: true,
        configurable: true
    });
    ///
    /// Public methods
    ///
    /**
     * Called on after component view as been initialized. Part of the ng Component life cycle.
     *
     * @returns {void}
     *
     * @memberof MapGeoTrackerComponent
     */
    MapGeoTrackerComponent.prototype.ngOnInit = function () {
        this._subscription = this._mapService.SubscribeToMapEvent('mousemove').subscribe(function (e) {
            console.log(e);
            //this.Track({
            //    latitude: e.latLng.lat(),
            //    longitude: e.latLng.lng()
            //});
        });
    };
    /**
     * Called on component destruction. Frees the resources used by the component. Part of the ng Component life cycle.
     *
     * @returns {void}
     *
     * @memberof MMapGeoTrackerComponentap
     */
    MapGeoTrackerComponent.prototype.ngOnDestroy = function () {
        if (this._subscription) {
            this._subscription.unsubscribe();
            this._subscription = null;
        }
    };
    ///
    /// Private methods
    ///
    /**
     * Updates the tracker coordinates.
     *
     * @private
     * @param {ILatLong} position - The position to update.
     * @memberof MapComponent
     */
    MapGeoTrackerComponent.prototype.Track = function (position) {
        this._position = position;
    };
    MapGeoTrackerComponent.decorators = [
        { type: core_1.Component, args: [{
                    selector: 'x-geo-tracker',
                    template: "\n        <div #tracker *ngIf=\"Position != null\" class=\"geo-tracker\">\n            <span class=\"lat\">{{Position.latitude}}&#9702;</span>\n            <span class=\"lng\">{{Position.longitude}}&#9702;</span>\n        </div>",
                    styles: ["\n        .geo-tracker { z-index: 1000; position: absolute; top: 50px; left: 50px; }\n        .geo-tracker .lat { }\n        .geo-tracker .lng { }\n    "],
                },] },
    ];
    /** @nocollapse */
    MapGeoTrackerComponent.ctorParameters = function () { return [
        { type: map_service_1.MapService, },
    ]; };
    MapGeoTrackerComponent.propDecorators = {
        'xOffset': [{ type: core_1.Input },],
        'yOffset': [{ type: core_1.Input },],
        'Attachment': [{ type: core_1.Input },],
    };
    return MapGeoTrackerComponent;
}());
exports.MapGeoTrackerComponent = MapGeoTrackerComponent;
/**
 * Enumeration defining the attachment position of the tracker on the map.
 */
var Attachment;
(function (Attachment) {
    Attachment[Attachment["TopLeft"] = 0] = "TopLeft";
    Attachment[Attachment["TopCenter"] = 1] = "TopCenter";
    Attachment[Attachment["TopRight"] = 2] = "TopRight";
    Attachment[Attachment["MiddleLeft"] = 3] = "MiddleLeft";
    Attachment[Attachment["MiddleRight"] = 4] = "MiddleRight";
    Attachment[Attachment["BottomLeft"] = 5] = "BottomLeft";
    Attachment[Attachment["BottomCenter"] = 6] = "BottomCenter";
    Attachment[Attachment["BottomRight"] = 7] = "BottomRight";
})(Attachment = exports.Attachment || (exports.Attachment = {}));
//# sourceMappingURL=map-geo-tracker.js.map