"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var marker_service_1 = require("../services/marker.service");
var layer_service_1 = require("../services/layer.service");
var cluster_service_1 = require("../services/cluster.service");
var map_service_1 = require("../services/map.service");
var cluster_click_action_1 = require("../models/cluster-click-action");
var cluster_placement_mode_1 = require("../models/cluster-placement-mode");
var cluster_layer_1 = require("./cluster-layer");
/**
 * internal counter to use as ids for marker.
 */
var layerId = 1000000;
/**
 * MapMarkerLayerDirective performantly renders a large set of map marker inside a {@link MapComponent}.
 *
 * ### Example
 * ```typescript
 * import {Component} from '@angular/core';
 * import {MapComponent, MapMarkerDirective} from '...';
 *
 * @Component({
 *  selector: 'my-map-cmp',
 *  styles: [`
 *   .map-container {
 *     height: 300px;
 *   }
 * `],
 * template: `
 *   <x-map [Latitude]="lat" [Longitude]="lng" [Zoom]="zoom">
 *      <x-map-marker-layer [MarkerOptions]="_markers"></x-map-marker-layer>
 *   </x-map>
 * `
 * })
 * ```
 *
 * @export
 * @class MapMarkerLayerDirective
 * @implements {OnDestroy}
 * @implements {OnChanges}
 * @implements {AfterContentInit}
 */
var MapMarkerLayerDirective = /** @class */ (function () {
    ///
    /// Constructor
    ///
    /**
     * Creates an instance of MapMarkerLayerDirective.
     * @param {MarkerService} _markerService - Concreate implementation of a {@link MarkerService}.
     * @param {LayerService} _layerService - Concreate implementation of a {@link LayerService}.
     * @param {ClusterService} _clusterService - Concreate implementation of a {@link ClusterService}.
     * @param {MapService} _mapService - Concreate implementation of a {@link MapService}.
     * @param {NgZone} _zone - Concreate implementation of a {@link NgZone} service.
     *
     * @memberof MapMarkerLayerDirective
     */
    function MapMarkerLayerDirective(_markerService, _layerService, _clusterService, _mapService, _zone) {
        this._markerService = _markerService;
        this._layerService = _layerService;
        this._clusterService = _clusterService;
        this._mapService = _mapService;
        this._zone = _zone;
        this._useDynamicSizeMarker = false;
        this._dynamicMarkerBaseSize = 18;
        this._dynamicMarkerRanges = new Map([
            [10, 'rgba(20, 180, 20, 0.5)'],
            [100, 'rgba(255, 210, 40, 0.5)'],
            [Number.MAX_SAFE_INTEGER, 'rgba(255, 40, 40, 0.5)']
        ]);
        this._streaming = false;
        this._markers = new Array();
        this._markersLast = new Array();
        /**
         * Gets or sets the the Cluster Click Action {@link ClusterClickAction}.
         *
         * @type {ClusterClickAction}
         * @memberof MapMarkerLayerDirective
         */
        this.ClusterClickAction = cluster_click_action_1.ClusterClickAction.ZoomIntoCluster;
        /**
         * Gets or sets the cluster placement mode. {@link ClusterPlacementMode}
         *
         * @type {ClusterPlacementMode}
         * @memberof MapMarkerLayerDirective
         */
        this.ClusterPlacementMode = cluster_placement_mode_1.ClusterPlacementMode.MeanValue;
        /**
         *  Determines whether the layer clusters. This property can only be set on creation of the layer.
         *
         * @type {Boolean}
         * @memberof MapMarkerLayerDirective
         */
        this.EnableClustering = false;
        /**
         * Gets or sets the grid size to be used for clustering.
         *
         * @type {number}
         * @memberof MapMarkerLayerDirective
         */
        this.GridSize = 150;
        /**
         * Gets or sets An offset applied to the positioning of the layer.
         *
         * @type {IPoint}
         * @memberof MapMarkerLayerDirective
         */
        this.LayerOffset = null;
        /**
         * Gets or sets the z-index of the layer. If not used, layers get stacked in the order created.
         *
         * @type {number}
         * @memberof MapMarkerLayerDirective
         */
        this.ZIndex = 0;
        /**
         * Gets or sets whether the cluster should zoom in on click
         *
         * @readonly
         * @type {boolean}
         * @memberof MapMarkerLayerDirective
         */
        this.ZoomOnClick = true;
        ///
        /// Delegates
        ///
        /**
         * This event emitter gets emitted when the dynamic icon for a marker is being created.
         *
         * @type {EventEmitter<IMarkerIconInfo>}
         * @memberof MapMarkerLayerDirective
         */
        this.DynamicMarkerCreated = new core_1.EventEmitter();
        /**
         * This event emitter gets emitted when the user clicks a marker in the layer.
         *
         * @type {EventEmitter<IMarkerEvent>}
         * @memberof MapMarkerLayerDirective
         */
        this.MarkerClick = new core_1.EventEmitter();
        /**
         * This event is fired when the user stops dragging a marker.
         *
         * @type {EventEmitter<IMarkerEvent>}
         * @memberof MapMarkerLayerDirective
         */
        this.DragEnd = new core_1.EventEmitter();
        this._id = layerId++;
    }
    Object.defineProperty(MapMarkerLayerDirective.prototype, "CustomMarkerCallback", {
        /**
         * Gets or sets the callback invoked to create a custom cluster marker. Note that when {@link UseDynamicSizeMarkers} is enabled,
         * you cannot set a custom marker callback.
         *
         * @type (Array<Marker>, IMarkerIconInfo) => string
         * @memberof MapMarkerLayerDirective
         */
        get: function () { return this._iconCreationCallback; },
        set: function (val) {
            if (this._useDynamicSizeMarker) {
                throw (new Error("You cannot set a custom marker callback when UseDynamicSizeMarkers is set to true.\n                    Set UseDynamicSizeMakers to false."));
            }
            this._iconCreationCallback = val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapMarkerLayerDirective.prototype, "DynamicMarkerBaseSize", {
        /**
         * Gets or sets the base size of dynamic markers in pixels. The actualy size of the dynamic marker is based on this.
         * See {@link UseDynamicSizeMarkers}.
         *
         * @type {number}
         * @memberof ClusterLayerDirective
         */
        get: function () { return this._dynamicMarkerBaseSize; },
        set: function (val) { this._dynamicMarkerBaseSize = val; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapMarkerLayerDirective.prototype, "DynamicMarkerRanges", {
        /**
         * Gets or sets the ranges to use to calculate breakpoints and colors for dynamic markers.
         * The map contains key/value pairs, with the keys being
         * the breakpoint sizes and the values the colors to be used for the dynamic marker in that range. See {@link UseDynamicSizeMarkers}.
         *
         * @type {Map<number, string>}
         * @memberof ClusterLayerDirective
         */
        get: function () { return this._dynamicMarkerRanges; },
        set: function (val) { this._dynamicMarkerRanges = val; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapMarkerLayerDirective.prototype, "MarkerOptions", {
        /**
         *  IMarkerOptions array holding the marker info.
         *
         * @type {Array<IMarkerOptions>}
         * @memberof MapMarkerLayerDirective
         */
        get: function () { return this._markers; },
        set: function (val) {
            if (this._streaming) {
                this._markersLast = val.slice(0);
                (_a = this._markers).push.apply(_a, val);
            }
            else {
                this._markers = val.slice(0);
            }
            var _a;
        },
        enumerable: true,
        configurable: true
    });
    ;
    ;
    Object.defineProperty(MapMarkerLayerDirective.prototype, "Styles", {
        /**
         * Gets or sets the cluster styles
         *
         * @type {Array<IClusterIconInfo>)}
         * @memberof MapMarkerLayerDirective
         */
        get: function () { return this._styles; },
        set: function (val) { this._styles = val; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapMarkerLayerDirective.prototype, "TreatNewMarkerOptionsAsStream", {
        /**
         * Sets whether to treat changes in the MarkerOptions as streams of new markers. In thsi mode, changing the
         * Array supplied in MarkerOptions will be incrementally drawn on the map as opposed to replace the markers on the map.
         *
         * @type {boolean}
         * @memberof MapMarkerLayerDirective
         */
        get: function () { return this._streaming; },
        set: function (val) { this._streaming = val; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapMarkerLayerDirective.prototype, "UseDynamicSizeMarkers", {
        /**
         * Gets or sets whether to use dynamic markers. Dynamic markers change in size and color depending on the number of
         * pins in the cluster. If set to true, this will take precendence over any custom marker creation.
         *
         * @type {boolean}
         * @memberof MapMarkerLayerDirective
         */
        get: function () { return this._useDynamicSizeMarker; },
        set: function (val) {
            var _this = this;
            this._useDynamicSizeMarker = val;
            if (val) {
                this._iconCreationCallback = function (m, info) {
                    return cluster_layer_1.ClusterLayerDirective.CreateDynamicSizeMarker(m.length, info, _this._dynamicMarkerBaseSize, _this._dynamicMarkerRanges);
                };
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapMarkerLayerDirective.prototype, "Id", {
        ///
        /// Property declarations
        ///
        /**
         * Gets the id of the marker layer.
         *
         * @readonly
         * @type {number}
         * @memberof MapMarkerLayerDirective
         */
        get: function () { return this._id; },
        enumerable: true,
        configurable: true
    });
    ///
    /// Public methods
    ///
    /**
     * Translates a geo location to a pixel location relative to the map viewport.
     *
     * @param {ILatLong} [loc] - {@link ILatLong} containing the geo coordinates.
     * @returns {Promise<IPoint>} - A promise that when fullfilled contains an {@link IPoint} representing the pixel coordinates.
     *
     * @memberof MapMarkerLayerDirective
     */
    MapMarkerLayerDirective.prototype.LocationToPixel = function (loc) {
        return this._markerService.LocationToPoint(loc);
    };
    /**
     * Called after Component content initialization. Part of ng Component life cycle.
     * @returns {void}
     *
     * @memberof MapMarkerLayerDirective
     */
    MapMarkerLayerDirective.prototype.ngAfterContentInit = function () {
        var _this = this;
        var layerOptions = {
            id: this._id
        };
        this._zone.runOutsideAngular(function () {
            var fakeLayerDirective = {
                Id: _this._id,
                Visible: _this.Visible,
                LayerOffset: _this.LayerOffset,
                ZIndex: _this.ZIndex
            };
            if (!_this.EnableClustering) {
                _this._layerService.AddLayer(fakeLayerDirective);
                _this._layerPromise = _this._layerService.GetNativeLayer(fakeLayerDirective);
                _this._service = _this._layerService;
            }
            else {
                fakeLayerDirective.ClusteringEnabled = _this.EnableClustering;
                fakeLayerDirective.ClusterPlacementMode = _this.ClusterPlacementMode;
                fakeLayerDirective.GridSize = _this.GridSize;
                fakeLayerDirective.ClusterClickAction = _this.ClusterClickAction;
                fakeLayerDirective.IconInfo = _this.ClusterIconInfo;
                fakeLayerDirective.CustomMarkerCallback = _this.CustomMarkerCallback;
                fakeLayerDirective.UseDynamicSizeMarkers = _this.UseDynamicSizeMarkers;
                _this._clusterService.AddLayer(fakeLayerDirective);
                _this._layerPromise = _this._clusterService.GetNativeLayer(fakeLayerDirective);
                _this._service = _this._clusterService;
            }
            if (_this.MarkerOptions) {
                _this._zone.runOutsideAngular(function () { return _this.UpdateMarkers(); });
            }
        });
    };
    /**
     * Called on component destruction. Frees the resources used by the component. Part of the ng Component life cycle.
     *
     *
     * @memberof MapMarkerLayerDirective
     */
    MapMarkerLayerDirective.prototype.ngOnDestroy = function () {
        this._layerPromise.then(function (l) {
            l.Delete();
        });
    };
    /**
     * Reacts to changes in data-bound properties of the component and actuates property changes in the underling layer model.
     *
     * @param {{ [propName: string]: SimpleChange }} changes - collection of changes.
     *
     * @memberof MapMarkerLayerDirective
     */
    MapMarkerLayerDirective.prototype.ngOnChanges = function (changes) {
        var _this = this;
        var shouldSetOptions = false;
        var o = {
            id: this._id
        };
        if (changes['MarkerOptions']) {
            this._zone.runOutsideAngular(function () {
                _this.UpdateMarkers();
            });
        }
        if (changes['Visible'] && !changes['Visible'].firstChange) {
            this._zone.runOutsideAngular(function () {
                _this._layerPromise.then(function (l) { return l.SetVisible(_this.Visible); });
            });
        }
        if (changes['EnableClustering'] && !changes['EnableClustering'].firstChange) {
            if ('StopClustering' in this._service) {
                o.clusteringEnabled = this.EnableClustering;
                shouldSetOptions = true;
            }
            else {
                throw (new Error('You cannot change EnableClustering after the layer has been created.'));
            }
        }
        if (changes['ClusterPlacementMode'] && !changes['ClusterPlacementMode'].firstChange && 'StopClustering' in this._service) {
            o.placementMode = this.ClusterPlacementMode;
            shouldSetOptions = true;
        }
        if (changes['GridSize'] && !changes['GridSize'].firstChange && 'StopClustering' in this._service) {
            o.gridSize = this.GridSize;
            shouldSetOptions = true;
        }
        if (changes['ClusterClickAction'] && !changes['ClusterClickAction'].firstChange && 'StopClustering' in this._service) {
            o.zoomOnClick = this.ClusterClickAction === cluster_click_action_1.ClusterClickAction.ZoomIntoCluster;
            shouldSetOptions = true;
        }
        if ((changes['ZIndex'] && !changes['ZIndex'].firstChange) ||
            (changes['LayerOffset'] && !changes['LayerOffset'].firstChange) ||
            (changes['IconInfo'] && !changes['IconInfo'].firstChange)) {
            throw (new Error('You cannot change ZIndex or LayerOffset after the layer has been created.'));
        }
        if (shouldSetOptions) {
            this._zone.runOutsideAngular(function () {
                var fakeLayerDirective = { Id: _this._id };
                _this._layerPromise.then(function (l) { return l.SetOptions(o); });
            });
        }
    };
    /**
     * Obtains a string representation of the Marker Id.
     * @return {string} - string representation of the marker id.
     * @memberof MapMarkerLayerDirective
     */
    MapMarkerLayerDirective.prototype.toString = function () { return 'MapMarkerLayer-' + this._id.toString(); };
    ///
    /// Private methods
    ///
    /**
     * Adds various event listeners for the marker.
     *
     * @param {Marker} m - the marker for which to add the event.
     * @private
     *
     * @memberof MapMarkerLayerDirective
     */
    MapMarkerLayerDirective.prototype.AddEventListeners = function (m) {
        var _this = this;
        m.AddListener('click', function (e) { return _this.MarkerClick.emit({
            Marker: m,
            Click: e,
            Location: _this._markerService.GetCoordinatesFromClick(e),
            Pixels: _this._markerService.GetPixelsFromClick(e)
        }); });
        m.AddListener('dragend', function (e) { return _this.DragEnd.emit({
            Marker: m,
            Click: e,
            Location: _this._markerService.GetCoordinatesFromClick(e),
            Pixels: _this._markerService.GetPixelsFromClick(e)
        }); });
    };
    /**
     * Sets or updates the markers based on the marker options. This will place the markers on the map
     * and register the associated events.
     *
     * @memberof MapMarkerLayerDirective
     * @method
     * @private
     */
    MapMarkerLayerDirective.prototype.UpdateMarkers = function () {
        var _this = this;
        if (this._layerPromise == null) {
            return;
        }
        this._layerPromise.then(function (l) {
            var markers = _this._streaming ? _this._markersLast : _this._markers;
            if (_this.Visible === false) {
                markers.forEach(function (o) { return o.visible = false; });
            }
            // generate the promise for the markers
            var mp = _this._service.CreateMarkers(markers, _this.IconInfo);
            // set markers once promises are fullfilled.
            mp.then(function (m) {
                m.forEach(function (marker) {
                    _this.AddEventListeners(marker);
                });
                _this._streaming ? l.AddEntities(m) : l.SetEntities(m);
            });
        });
    };
    MapMarkerLayerDirective.decorators = [
        { type: core_1.Directive, args: [{
                    selector: 'x-map-marker-layer'
                },] },
    ];
    /** @nocollapse */
    MapMarkerLayerDirective.ctorParameters = function () { return [
        { type: marker_service_1.MarkerService, },
        { type: layer_service_1.LayerService, },
        { type: cluster_service_1.ClusterService, },
        { type: map_service_1.MapService, },
        { type: core_1.NgZone, },
    ]; };
    MapMarkerLayerDirective.propDecorators = {
        'ClusterClickAction': [{ type: core_1.Input },],
        'ClusterIconInfo': [{ type: core_1.Input },],
        'ClusterPlacementMode': [{ type: core_1.Input },],
        'CustomMarkerCallback': [{ type: core_1.Input },],
        'DynamicMarkerBaseSize': [{ type: core_1.Input },],
        'DynamicMarkerRanges': [{ type: core_1.Input },],
        'EnableClustering': [{ type: core_1.Input },],
        'GridSize': [{ type: core_1.Input },],
        'IconInfo': [{ type: core_1.Input },],
        'LayerOffset': [{ type: core_1.Input },],
        'MarkerOptions': [{ type: core_1.Input },],
        'Styles': [{ type: core_1.Input },],
        'TreatNewMarkerOptionsAsStream': [{ type: core_1.Input },],
        'UseDynamicSizeMarkers': [{ type: core_1.Input },],
        'Visible': [{ type: core_1.Input },],
        'ZIndex': [{ type: core_1.Input },],
        'ZoomOnClick': [{ type: core_1.Input },],
        'DynamicMarkerCreated': [{ type: core_1.Output },],
        'MarkerClick': [{ type: core_1.Output },],
        'DragEnd': [{ type: core_1.Output },],
    };
    return MapMarkerLayerDirective;
}());
exports.MapMarkerLayerDirective = MapMarkerLayerDirective;
//# sourceMappingURL=map-marker-layer.js.map