import { SimpleChange, OnDestroy, OnChanges, EventEmitter, AfterContentInit, ViewContainerRef } from '@angular/core';
import { IPoint } from '../interfaces/ipoint';
import { ILatLong } from '../interfaces/ilatlong';
import { IMarkerEvent } from '../interfaces/imarker-event';
import { IMarkerIconInfo } from '../interfaces/imarker-icon-info';
import { MarkerService } from '../services/marker.service';
import { InfoBoxComponent } from './infobox';
/**
 * MapMarkerDirective renders a map marker inside a {@link MapComponent}.
 *
 * ### Example
 * ```typescript
 * import {Component} from '@angular/core';
 * import {MapComponent, MapMarkerDirective} from '...';
 *
 * @Component({
 *  selector: 'my-map-cmp',
 *  styles: [`
 *   .map-container {
 *     height: 300px;
 *   }
 * `],
 * template: `
 *   <x-map [Latitude]="lat" [Longitude]="lng" [Zoom]="zoom">
 *      <x-map-marker [Latitude]="lat" [Longitude]="lng" [Label]="'M'"></x-map-marker>
 *   </x-map>
 * `
 * })
 * ```
 *
 * @export
 * @class MapMarkerDirective
 * @implements {OnDestroy}
 * @implements {OnChanges}
 * @implements {AfterContentInit}
 */
export declare class MapMarkerDirective implements OnDestroy, OnChanges, AfterContentInit {
    private _markerService;
    private _containerRef;
    private _inCustomLayer;
    private _inClusterLayer;
    private _markerAddedToManger;
    private _id;
    private _layerId;
    private _events;
    private _clickTimeout;
    /**
     * Any InfoBox that is a direct children of the marker
     *
     * @protected
     * @type {InfoBox}
     * @memberof MapMarkerDirective
     */
    protected _infoBox: InfoBoxComponent;
    /**
     *  Icon anchor relative to marker root
     *
     * @type {IPoint}
     * @memberof MapMarkerDirective
     */
    Anchor: IPoint;
    /**
     * If true, the marker can be dragged. Default value is false.
     *
     * @type {boolean}
     * @memberof MapMarkerDirective
     */
    Draggable: boolean;
    /**
     * Icon height
     *
     * @type {number}
     * @memberof MapMarkerDirective
     */
    Height: number;
    /**
     * Information for dynamic, custom created icons.
     *
     * @type {IMarkerIconInfo}
     * @memberof MapMarkerDirective
     */
    IconInfo: IMarkerIconInfo;
    /**
     * Icon (the URL of the image) for the foreground.
     *
     * @type {string}
     * @memberof MapMarkerDirective
     */
    IconUrl: string;
    /**
     * True to indiciate whether this is the first marker in a set.
     * Use this for bulk operations (particularily clustering) to ensure performance.
     *
     * @type {boolean}
     * @memberof MapMarkerDirective
     */
    IsFirstInSet: boolean;
    /**
     * True to indiciate whether this is the last marker in a set.
     * Use this for bulk operations (particularily clustering) to ensure performance.
     *
     * @type {boolean}
     * @memberof MapMarkerDirective
     */
    IsLastInSet: boolean;
    /**
     * The label (a single uppercase character) for the marker.
     *
     * @type {string}
     * @memberof MapMarkerDirective
     */
    Label: string;
    /**
     * The latitude position of the marker.
     *
     * @type {number}
     * @memberof MapMarkerDirective
     */
    Latitude: number;
    /**
     * The longitude position of the marker.
     *
     * @type {number}
     * @memberof MapMarkerDirective
     */
    Longitude: number;
    /**
     * Arbitary metadata to assign to the Marker. This is useful for events
     *
     * @type {Map<string, any>}
     * @memberof MapMarkerDirective
     */
    Metadata: Map<string, any>;
    /**
     *  The title of the marker.
     *
     * @type {string}
     * @memberof MapMarkerDirective
     */
    Title: string;
    /**
     * Sets the visibility of the marker
     *
     * @type {string}
     * @memberof MapMarkerDirective
     */
    Visible: boolean;
    /**
     * Icon Width
     *
     * @type {number}
     * @memberof MapMarkerDirective
     */
    Width: number;
    /**
     * This event emitter gets emitted when a marker icon is being created.
     *
     * @type {EventEmitter<IMarkerIconInfo>}
     * @memberof MapMarkerDirective
     */
    DynamicMarkerCreated: EventEmitter<IMarkerIconInfo>;
    /**
     * This event emitter gets emitted when the user clicks on the marker.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    MarkerClick: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired when the DOM dblclick event is fired on the marker.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    DblClick: EventEmitter<IMarkerEvent>;
    /**
     * This event is repeatedly fired while the user drags the marker.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    Drag: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired when the user stops dragging the marker.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    DragEnd: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired when the user starts dragging the marker.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    DragStart: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired when the DOM mousedown event is fired on the marker.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    MouseDown: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired when the DOM mousemove event is fired on the marker.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    MouseMove: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired on marker mouseout.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    MouseOut: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired on marker mouseover.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    MouseOver: EventEmitter<IMarkerEvent>;
    /**
     * This event is fired whe the DOM mouseup event is fired on the marker
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    MouseUp: EventEmitter<IMarkerEvent>;
    /**
     * This even is fired when the marker is right-clicked on.
     *
     * @type {EventEmitter<IMarkerEvent>}
     * @memberof MapMarkerDirective
     */
    RightClick: EventEmitter<IMarkerEvent>;
    /**
     * Getswhether the marker has already been added to the marker service and is ready for use.
     *
     * @readonly
     * @type {boolean}
     * @memberof MapMarkerDirective
     */
    readonly AddedToManager: boolean;
    /**
     * Gets the id of the marker as a string.
     *
     * @readonly
     * @type {string}
     * @memberof MapMarkerDirective
     */
    readonly Id: string;
    /**
     * Gets whether the marker is in a cluster layer. See {@link ClusterLayer}.
     *
     * @readonly
     * @type {boolean}
     * @memberof MapMarkerDirective
     */
    readonly InClusterLayer: boolean;
    /**
     * Gets whether the marker is in a custom layer. See {@link MapLayer}.
     *
     * @readonly
     * @type {boolean}
     * @memberof MapMarkerDirective
     */
    readonly InCustomLayer: boolean;
    /**
     * gets the id of the Layer the marker belongs to.
     *
     * @readonly
     * @type {number}
     * @memberof MapMarkerDirective
     */
    readonly LayerId: number;
    /**
     * Creates an instance of MapMarkerDirective.
     * @param {MarkerService} _markerService - Concreate implementation of a {@link MarkerService}.
     * @param {ViewContainerRef} _containerRef - View container hosting the marker.
     * Used to determine parent layer through markup.
     *
     * @memberof MapMarkerDirective
     */
    constructor(_markerService: MarkerService, _containerRef: ViewContainerRef);
    /**
     * Translates a marker geo location to a pixel location relative to the map viewport.
     *
     * @param {ILatLong} [loc] - {@link ILatLong} containing the geo coordinates. If null, the marker's coordinates are used.
     * @returns {Promise<IPoint>} - A promise that when fullfilled contains an {@link IPoint} representing the pixel coordinates.
     *
     * @memberof MapMarkerDirective
     */
    LocationToPixel(loc?: ILatLong): Promise<IPoint>;
    /**
     * Called after Component content initialization. Part of ng Component life cycle.
     * @returns {void}
     *
     * @memberof MapMarkerDirective
     */
    ngAfterContentInit(): void;
    /**
     * Called on component destruction. Frees the resources used by the component. Part of the ng Component life cycle.
     *
     *
     * @memberof MapMarkerDirective
     */
    ngOnDestroy(): void;
    /**
     * Reacts to changes in data-bound properties of the component and actuates property changes in the underling layer model.
     *
     * @param {{ [propName: string]: SimpleChange }} changes - collection of changes.
     *
     * @memberof MapMarkerDirective
     */
    ngOnChanges(changes: {
        [key: string]: SimpleChange;
    }): void;
    /**
     * Obtains a string representation of the Marker Id.
     * @return {string} - string representation of the marker id.
     * @memberof MapMarkerDirective
     */
    toString(): string;
    /**
     * Adds various event listeners for the marker.
     *
     * @private
     *
     * @memberof MapMarkerDirective
     */
    private AddEventListeners();
}
