"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var layer_service_1 = require("../services/layer.service");
var map_service_1 = require("../services/map.service");
/**
 * internal counter to use as ids for polygons.
 */
var layerId = 1000000;
/**
 * MapPolygonLayerDirective performantly renders a large set of polygons on a {@link MapComponent}.
 *
 * ### Example
 * ```typescript
 * import {Component} from '@angular/core';
 * import {MapComponent} from '...';
 *
 * @Component({
 *  selector: 'my-map-cmp',
 *  styles: [`
 *   .map-container {
 *     height: 300px;
 *   }
 * `],
 * template: `
 *   <x-map [Latitude]="lat" [Longitude]="lng" [Zoom]="zoom">
 *      <x-map-polygon-layer [PolygonOptions]="_polygons"></x-map-polygon-layer>
 *   </x-map>
 * `
 * })
 * ```
 *
 * @export
 * @class MapPolygonLayerDirective
 * @implements {OnDestroy}
 * @implements {OnChanges}
 * @implements {AfterContentInit}
 */
var MapPolygonLayerDirective = /** @class */ (function () {
    ///
    /// Constructor
    ///
    /**
     * Creates an instance of MapPolygonLayerDirective.
     * @param {LayerService} _layerService - Concreate implementation of a {@link LayerService}.
     * @param {MapService} _mapService - Concreate implementation of a {@link MapService}.
     * @param {NgZone} _zone - Concreate implementation of a {@link NgZone} service.
     * @memberof MapPolygonLayerDirective
     */
    function MapPolygonLayerDirective(_layerService, _mapService, _zone) {
        this._layerService = _layerService;
        this._mapService = _mapService;
        this._zone = _zone;
        this._labels = new Array();
        this._tooltipSubscriptions = new Array();
        this._tooltipVisible = false;
        this._defaultOptions = {
            fontSize: 11,
            fontFamily: 'sans-serif',
            strokeWeight: 2,
            strokeColor: '#000000',
            fontColor: '#ffffff'
        };
        this._streaming = false;
        this._polygons = new Array();
        this._polygonsLast = new Array();
        /**
         * Set the maximum zoom at which the polygon labels are visible. Ignored if ShowLabel is false.
         * @type {number}
         * @memberof MapPolygonLayerDirective
         */
        this.LabelMaxZoom = Number.MAX_SAFE_INTEGER;
        /**
         * Set the minimum zoom at which the polygon labels are visible. Ignored if ShowLabel is false.
         * @type {number}
         * @memberof MapPolygonLayerDirective
         */
        this.LabelMinZoom = -1;
        /**
         * Gets or sets An offset applied to the positioning of the layer.
         *
         * @type {IPoint}
         * @memberof MapPolygonLayerDirective
         */
        this.LayerOffset = null;
        /**
         * Whether to show the polygon titles as the labels on the polygons.
         *
         * @type {boolean}
         * @memberof MapPolygonLayerDirective
         */
        this.ShowLabels = false;
        /**
         * Whether to show the titles of the polygosn as the tooltips on the polygons.
         *
         * @type {boolean}
         * @memberof MapPolygonLayerDirective
         */
        this.ShowTooltips = true;
        /**
         * Gets or sets the z-index of the layer. If not used, layers get stacked in the order created.
         *
         * @type {number}
         * @memberof MapPolygonLayerDirective
         */
        this.ZIndex = 0;
        ///
        /// Delegates
        ///
        /**
         * This event emitter gets emitted when the user clicks a polygon in the layer.
         *
         * @type {EventEmitter<IPolygonEvent>}
         * @memberof MapPolygonLayerDirective
         */
        this.PolygonClick = new core_1.EventEmitter();
        /**
         * This event is fired when the DOM dblclick event is fired on a polygon in the layer.
         *
         * @type {EventEmitter<IPolygonEvent>}
         * @memberof MapPolygonLayerDirective
         */
        this.PolygonDblClick = new core_1.EventEmitter();
        /**
         * This event is fired when the DOM mousemove event is fired on a polygon in the layer.
         *
         * @type {EventEmitter<IPolygonEvent>}
         * @memberof MapPolygonLayerDirective
         */
        this.PolygonMouseMove = new core_1.EventEmitter();
        /**
         * This event is fired on mouseout on a polygon in the layer.
         *
         * @type {EventEmitter<IPolygonEvent>}
         * @memberof MapPolygonLayerDirective
         */
        this.PolygonMouseOut = new core_1.EventEmitter();
        /**
         * This event is fired on mouseover on a polygon in a layer.
         *
         * @type {EventEmitter<IPolygonEvent>}
         * @memberof MapPolygonLayerDirective
         */
        this.PolygonMouseOver = new core_1.EventEmitter();
        this._id = layerId++;
    }
    Object.defineProperty(MapPolygonLayerDirective.prototype, "PolygonOptions", {
        /**
         * An array of polygon options representing the polygons in the layer.
         *
         * @type {Array<IPolygonOptions>}
         * @memberof MapPolygonLayerDirective
         */
        get: function () { return this._polygons; },
        set: function (val) {
            if (this._streaming) {
                this._polygonsLast = val.slice(0);
                (_a = this._polygons).push.apply(_a, val);
            }
            else {
                this._polygons = val.slice(0);
            }
            var _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapPolygonLayerDirective.prototype, "TreatNewPolygonOptionsAsStream", {
        /**
         * Sets whether to treat changes in the PolygonOptions as streams of new markers. In this mode, changing the
         * Array supplied in PolygonOptions will be incrementally drawn on the map as opposed to replace the polygons on the map.
         *
         * @type {boolean}
         * @memberof MapPolygonLayerDirective
         */
        get: function () { return this._streaming; },
        set: function (val) { this._streaming = val; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapPolygonLayerDirective.prototype, "Id", {
        ///
        /// Property declarations
        ///
        /**
         * Gets the id of the marker layer.
         *
         * @readonly
         * @type {number}
         * @memberof MapPolygonLayerDirective
         */
        get: function () { return this._id; },
        enumerable: true,
        configurable: true
    });
    ///
    /// Public methods
    ///
    /**
     * Called after Component content initialization. Part of ng Component life cycle.
     * @returns {void}
     *
     * @memberof MapPolygonLayerDirective
     */
    MapPolygonLayerDirective.prototype.ngAfterContentInit = function () {
        var _this = this;
        var layerOptions = {
            id: this._id
        };
        this._zone.runOutsideAngular(function () {
            var fakeLayerDirective = {
                Id: _this._id,
                Visible: _this.Visible,
                LayerOffset: _this.LayerOffset,
                ZIndex: _this.ZIndex
            };
            _this._layerService.AddLayer(fakeLayerDirective);
            _this._layerPromise = _this._layerService.GetNativeLayer(fakeLayerDirective);
            _this._mapService.CreateCanvasOverlay(function (el) { return _this.DrawLabels(el); }).then(function (c) {
                _this._canvas = c;
                c._canvasReady.then(function (b) {
                    _this._tooltip = c.GetToolTipOverlay();
                    _this.ManageTooltip(_this.ShowTooltips);
                });
                if (_this.PolygonOptions) {
                    _this._zone.runOutsideAngular(function () { return _this.UpdatePolygons(); });
                }
            });
            _this._service = _this._layerService;
        });
    };
    /**
     * Called on component destruction. Frees the resources used by the component. Part of the ng Component life cycle.
     *
     * @memberof MapPolygonLayerDirective
     */
    MapPolygonLayerDirective.prototype.ngOnDestroy = function () {
        this._tooltipSubscriptions.forEach(function (s) { return s.unsubscribe(); });
        this._layerPromise.then(function (l) {
            l.Delete();
        });
        if (this._canvas) {
            this._canvas.Delete();
        }
    };
    /**
     * Reacts to changes in data-bound properties of the component and actuates property changes in the underling layer model.
     *
     * @param {{ [propName: string]: SimpleChange }} changes - collection of changes.
     * @memberof MapPolygonLayerDirective
     */
    MapPolygonLayerDirective.prototype.ngOnChanges = function (changes) {
        var _this = this;
        if (changes['PolygonOptions']) {
            this._zone.runOutsideAngular(function () {
                _this.UpdatePolygons();
            });
        }
        if (changes['Visible'] && !changes['Visible'].firstChange) {
            this._layerPromise.then(function (l) { return l.SetVisible(_this.Visible); });
        }
        if ((changes['ZIndex'] && !changes['ZIndex'].firstChange) ||
            (changes['LayerOffset'] && !changes['LayerOffset'].firstChange)) {
            throw (new Error('You cannot change ZIndex or LayerOffset after the layer has been created.'));
        }
        if ((changes['ShowLabels'] && !changes['ShowLabels'].firstChange) ||
            (changes['LabelMinZoom'] && !changes['LabelMinZoom'].firstChange) ||
            (changes['LabelMaxZoom'] && !changes['LabelMaxZoom'].firstChange)) {
            if (this._canvas) {
                this._canvas.Redraw(true);
            }
        }
        if (changes['ShowTooltips'] && this._tooltip) {
            this.ManageTooltip(changes['ShowTooltips'].currentValue);
        }
    };
    /**
     * Obtains a string representation of the Marker Id.
     * @return {string} - string representation of the marker id.
     * @memberof MapPolygonLayerDirective
     */
    MapPolygonLayerDirective.prototype.toString = function () { return 'MapPolygonLayer-' + this._id.toString(); };
    ///
    /// Private methods
    ///
    /**
     * Adds various event listeners for the marker.
     *
     * @param {Polygon} p - the polygon for which to add the event.
     * @private
     *
     * @memberof MapPolygonLayerDirective
     */
    MapPolygonLayerDirective.prototype.AddEventListeners = function (p) {
        var _this = this;
        var handlers = [
            { name: 'click', handler: function (ev) { return _this.PolygonClick.emit({ Polygon: p, Click: ev }); } },
            { name: 'dblclick', handler: function (ev) { return _this.PolygonDblClick.emit({ Polygon: p, Click: ev }); } },
            { name: 'mousemove', handler: function (ev) { return _this.PolygonMouseMove.emit({ Polygon: p, Click: ev }); } },
            { name: 'mouseout', handler: function (ev) { return _this.PolygonMouseOut.emit({ Polygon: p, Click: ev }); } },
            { name: 'mouseover', handler: function (ev) { return _this.PolygonMouseOver.emit({ Polygon: p, Click: ev }); } }
        ];
        handlers.forEach(function (obj) { return p.AddListener(obj.name, obj.handler); });
    };
    /**
     * Draws the polygon labels. Called by the Canvas overlay.
     *
     * @private
     * @param {HTMLCanvasElement} el - The canvas on which to draw the labels.
     * @memberof MapPolygonLayerDirective
     */
    MapPolygonLayerDirective.prototype.DrawLabels = function (el) {
        var _this = this;
        if (this.ShowLabels) {
            this._mapService.GetZoom().then(function (z) {
                if (_this.LabelMinZoom <= z && _this.LabelMaxZoom >= z) {
                    var ctx_1 = el.getContext('2d');
                    var labels_1 = _this._labels.map(function (x) { return x.title; });
                    _this._mapService.LocationsToPoints(_this._labels.map(function (x) { return x.loc; })).then(function (locs) {
                        var size = _this._mapService.MapSize;
                        for (var i = 0, len = locs.length; i < len; i++) {
                            // Don't draw the point if it is not in view. This greatly improves performance when zoomed in.
                            if (locs[i].x >= 0 && locs[i].y >= 0 && locs[i].x <= size.width && locs[i].y <= size.height) {
                                _this.DrawText(ctx_1, locs[i], labels_1[i]);
                            }
                        }
                    });
                }
            });
        }
    };
    /**
     * Draws the label text at the appropriate place on the canvas.
     * @param {CanvasRenderingContext2D} ctx - Canvas drawing context.
     * @param {IPoint} loc - Pixel location on the canvas where to center the text.
     * @param {string} text - Text to draw.
     */
    MapPolygonLayerDirective.prototype.DrawText = function (ctx, loc, text) {
        var lo = this.LabelOptions;
        if (lo == null && this._tooltip) {
            lo = this._tooltip.DefaultLabelStyle;
        }
        if (lo == null) {
            lo = this._defaultOptions;
        }
        ctx.strokeStyle = lo.strokeColor;
        ctx.font = lo.fontSize + "px " + lo.fontFamily;
        ctx.textAlign = 'center';
        var strokeWeight = lo.strokeWeight;
        if (text && strokeWeight && strokeWeight > 0) {
            ctx.lineWidth = strokeWeight;
            ctx.strokeText(text, loc.x, loc.y);
        }
        ctx.fillStyle = lo.fontColor;
        ctx.fillText(text, loc.x, loc.y);
    };
    /**
     * Manages the tooltip and the attachment of the associated events.
     *
     * @private
     * @param {boolean} show - True to enable the tooltip, false to disable.
     * @memberof MapPolygonLayerDirective
     */
    MapPolygonLayerDirective.prototype.ManageTooltip = function (show) {
        var _this = this;
        if (show && this._canvas) {
            // add tooltip subscriptions
            this._tooltip.Set('hidden', true);
            this._tooltipVisible = false;
            this._tooltipSubscriptions.push(this.PolygonMouseMove.asObservable().subscribe(function (e) {
                if (_this._tooltipVisible) {
                    var loc = _this._canvas.GetCoordinatesFromClick(e.Click);
                    _this._tooltip.Set('position', loc);
                }
            }));
            this._tooltipSubscriptions.push(this.PolygonMouseOver.asObservable().subscribe(function (e) {
                if (e.Polygon.Title && e.Polygon.Title.length > 0) {
                    var loc = _this._canvas.GetCoordinatesFromClick(e.Click);
                    _this._tooltip.Set('text', e.Polygon.Title);
                    _this._tooltip.Set('position', loc);
                    if (!_this._tooltipVisible) {
                        _this._tooltip.Set('hidden', false);
                        _this._tooltipVisible = true;
                    }
                }
            }));
            this._tooltipSubscriptions.push(this.PolygonMouseOut.asObservable().subscribe(function (e) {
                if (_this._tooltipVisible) {
                    _this._tooltip.Set('hidden', true);
                    _this._tooltipVisible = false;
                }
            }));
        }
        else {
            // remove tooltip subscriptions
            this._tooltipSubscriptions.forEach(function (s) { return s.unsubscribe(); });
            this._tooltipSubscriptions.splice(0);
            this._tooltip.Set('hidden', true);
            this._tooltipVisible = false;
        }
    };
    /**
     * Sets or updates the polygons based on the polygon options. This will place the polygons on the map
     * and register the associated events.
     *
     * @memberof MapPolygonLayerDirective
     * @method
     * @private
     */
    MapPolygonLayerDirective.prototype.UpdatePolygons = function () {
        var _this = this;
        if (this._layerPromise == null) {
            return;
        }
        this._layerPromise.then(function (l) {
            var polygons = _this._streaming ? _this._polygonsLast : _this._polygons;
            if (!_this._streaming) {
                _this._labels.splice(0);
            }
            if (_this.Visible === false) {
                _this.PolygonOptions.forEach(function (o) { return o.visible = false; });
            }
            // generate the promise for the markers
            var lp = _this._service.CreatePolygons(l.GetOptions().id, polygons);
            // set markers once promises are fullfilled.
            lp.then(function (p) {
                p.forEach(function (poly) {
                    if (poly.Title != null && poly.Title.length > 0) {
                        _this._labels.push({ loc: poly.Centroid, title: poly.Title });
                    }
                    _this.AddEventListeners(poly);
                });
                _this._streaming ? l.AddEntities(p) : l.SetEntities(p);
                if (_this._canvas) {
                    _this._canvas.Redraw(!_this._streaming);
                }
            });
        });
    };
    MapPolygonLayerDirective.decorators = [
        { type: core_1.Directive, args: [{
                    selector: 'x-map-polygon-layer'
                },] },
    ];
    /** @nocollapse */
    MapPolygonLayerDirective.ctorParameters = function () { return [
        { type: layer_service_1.LayerService, },
        { type: map_service_1.MapService, },
        { type: core_1.NgZone, },
    ]; };
    MapPolygonLayerDirective.propDecorators = {
        'LabelMaxZoom': [{ type: core_1.Input },],
        'LabelMinZoom': [{ type: core_1.Input },],
        'LabelOptions': [{ type: core_1.Input },],
        'LayerOffset': [{ type: core_1.Input },],
        'PolygonOptions': [{ type: core_1.Input },],
        'ShowLabels': [{ type: core_1.Input },],
        'ShowTooltips': [{ type: core_1.Input },],
        'TreatNewPolygonOptionsAsStream': [{ type: core_1.Input },],
        'Visible': [{ type: core_1.Input },],
        'ZIndex': [{ type: core_1.Input },],
        'PolygonClick': [{ type: core_1.Output },],
        'PolygonDblClick': [{ type: core_1.Output },],
        'PolygonMouseMove': [{ type: core_1.Output },],
        'PolygonMouseOut': [{ type: core_1.Output },],
        'PolygonMouseOver': [{ type: core_1.Output },],
    };
    return MapPolygonLayerDirective;
}());
exports.MapPolygonLayerDirective = MapPolygonLayerDirective;
//# sourceMappingURL=map-polygon-layer.js.map