import { SimpleChange, OnDestroy, OnChanges, EventEmitter, AfterContentInit, NgZone } from '@angular/core';
import { IPoint } from '../interfaces/ipoint';
import { IPolylineEvent } from '../interfaces/ipolyline-event';
import { IPolylineOptions } from '../interfaces/ipolyline-options';
import { ILabelOptions } from '../interfaces/ilabel-options';
import { LayerService } from '../services/layer.service';
import { MapService } from '../services/map.service';
/**
 * MapPolylineLayerDirective performantly renders a large set of polyline on a {@link MapComponent}.
 *
 * ### Example
 * ```typescript
 * import {Component} from '@angular/core';
 * import {MapComponent} from '...';
 *
 * @Component({
 *  selector: 'my-map-cmp',
 *  styles: [`
 *   .map-container {
 *     height: 300px;
 *   }
 * `],
 * template: `
 *   <x-map [Latitude]="lat" [Longitude]="lng" [Zoom]="zoom">
 *      <x-map-polyline-layer [PolygonOptions]="_polyline"></x-map-polyline-layer>
 *   </x-map>
 * `
 * })
 * ```
 *
 * @export
 * @class MapPolylineLayerDirective
 * @implements {OnDestroy}
 * @implements {OnChanges}
 * @implements {AfterContentInit}
 */
export declare class MapPolylineLayerDirective implements OnDestroy, OnChanges, AfterContentInit {
    private _layerService;
    private _mapService;
    private _zone;
    private _id;
    private _layerPromise;
    private _service;
    private _canvas;
    private _labels;
    private _tooltip;
    private _tooltipSubscriptions;
    private _tooltipVisible;
    private _defaultOptions;
    private _streaming;
    private _polylines;
    private _polylinesLast;
    /**
     * Set the maximum zoom at which the polyline labels are visible. Ignored if ShowLabel is false.
     * @type {number}
     * @memberof MapPolylineLayerDirective
     */
    LabelMaxZoom: number;
    /**
     * Set the minimum zoom at which the polyline labels are visible. Ignored if ShowLabel is false.
     * @type {number}
     * @memberof MapPolylineLayerDirective
     */
    LabelMinZoom: number;
    /**
     * Sepcifies styleing options for on-map polyline labels.
     *
     * @type {ILabelOptions}
     * @memberof MapPolylineLayerDirective
     */
    LabelOptions: ILabelOptions;
    /**
     * Gets or sets An offset applied to the positioning of the layer.
     *
     * @type {IPoint}
     * @memberof MapPolylineLayerDirective
     */
    LayerOffset: IPoint;
    /**
     * An array of polyline options representing the polylines in the layer.
     *
     * @type {Array<IPolylineOptions>}
     * @memberof MapPolylineLayerDirective
     */
    PolylineOptions: Array<IPolylineOptions>;
    /**
     * Whether to show the polylines titles as the labels on the polylines.
     *
     * @type {boolean}
     * @memberof MapPolylineLayerDirective
     */
    ShowLabels: boolean;
    /**
     * Whether to show the titles of the polylines as the tooltips on the polylines.
     *
     * @type {boolean}
     * @memberof MapPolylineLayerDirective
     */
    ShowTooltips: boolean;
    /**
     * Sets whether to treat changes in the PolylineOptions as streams of new markers. In this mode, changing the
     * Array supplied in PolylineOptions will be incrementally drawn on the map as opposed to replace the polylines on the map.
     *
     * @type {boolean}
     * @memberof MapPolylineLayerDirective
     */
    TreatNewPolylineOptionsAsStream: boolean;
    /**
     * Sets the visibility of the marker layer
     *
     * @type {string}
     * @memberof MapPolylineLayerDirective
     */
    Visible: boolean;
    /**
     * Gets or sets the z-index of the layer. If not used, layers get stacked in the order created.
     *
     * @type {number}
     * @memberof MapPolylineLayerDirective
     */
    ZIndex: number;
    /**
     * This event emitter gets emitted when the user clicks a polyline in the layer.
     *
     * @type {EventEmitter<IPolylineEvent>}
     * @memberof MapPolylineLayerDirective
     */
    PolylineClick: EventEmitter<IPolylineEvent>;
    /**
     * This event is fired when the DOM dblclick event is fired on a polyline in the layer.
     *
     * @type {EventEmitter<IPolylineEvent>}
     * @memberof MapPolylineLayerDirective
     */
    PolylineDblClick: EventEmitter<IPolylineEvent>;
    /**
     * This event is fired when the DOM mousemove event is fired on a polyline in the layer.
     *
     * @type {EventEmitter<IPolylineEvent>}
     * @memberof MapPolylineLayerDirective
     */
    PolylineMouseMove: EventEmitter<IPolylineEvent>;
    /**
     * This event is fired on mouseout on a polyline in the layer.
     *
     * @type {EventEmitter<IPolygonEvent>}
     * @memberof MapPolylineLayerDirective
     */
    PolylineMouseOut: EventEmitter<IPolylineEvent>;
    /**
     * This event is fired on mouseover on a polyline in a layer.
     *
     * @type {EventEmitter<IPolylineEvent>}
     * @memberof MapPolylineLayerDirective
     */
    PolylineMouseOver: EventEmitter<IPolylineEvent>;
    /**
     * Gets the id of the polyline layer.
     *
     * @readonly
     * @type {number}
     * @memberof MapPolylineLayerDirective
     */
    readonly Id: number;
    /**
     * Creates an instance of MapPolylineLayerDirective.
     * @param {LayerService} _layerService - Concreate implementation of a {@link LayerService}.
     * @param {MapService} _mapService - Concreate implementation of a {@link MapService}.
     * @param {NgZone} _zone - Concreate implementation of a {@link NgZone} service.
     * @memberof MapPolylineLayerDirective
     */
    constructor(_layerService: LayerService, _mapService: MapService, _zone: NgZone);
    /**
     * Called after Component content initialization. Part of ng Component life cycle.
     * @returns {void}
     *
     * @memberof MapPolylineLayerDirective
     */
    ngAfterContentInit(): void;
    /**
     * Called on component destruction. Frees the resources used by the component. Part of the ng Component life cycle.
     *
     * @memberof MapPolylineLayerDirective
     */
    ngOnDestroy(): void;
    /**
     * Reacts to changes in data-bound properties of the component and actuates property changes in the underling layer model.
     *
     * @param {{ [propName: string]: SimpleChange }} changes - collection of changes.
     * @memberof MapPolylineLayerDirective
     */
    ngOnChanges(changes: {
        [key: string]: SimpleChange;
    }): void;
    /**
     * Obtains a string representation of the Layer Id.
     * @return {string} - string representation of the layer id.
     * @memberof MapPolylineLayerDirective
     */
    toString(): string;
    /**
     * Adds various event listeners for the polylines.
     *
     * @param {Polyline} p - the polyline for which to add the event.
     * @private
     *
     * @memberof MapPolylineLayerDirective
     */
    private AddEventListeners(p);
    /**
     * Draws the polyline labels. Called by the Canvas overlay.
     *
     * @private
     * @param {HTMLCanvasElement} el - The canvas on which to draw the labels.
     * @memberof MapPolylineLayerDirective
     */
    private DrawLabels(el);
    /**
     * Draws the label text at the appropriate place on the canvas.
     * @param {CanvasRenderingContext2D} ctx - Canvas drawing context.
     * @param {IPoint} loc - Pixel location on the canvas where to center the text.
     * @param {string} text - Text to draw.
     */
    private DrawText(ctx, loc, text);
    /**
     * Manages the tooltip and the attachment of the associated events.
     *
     * @private
     * @param {boolean} show - True to enable the tooltip, false to disable.
     * @memberof MapPolygonLayerDirective
     */
    private ManageTooltip(show);
    /**
     * Sets or updates the polyliness based on the polyline options. This will place the polylines on the map
     * and register the associated events.
     *
     * @memberof MapPolylineLayerDirective
     * @method
     * @private
     */
    private UpdatePolylines();
}
