"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var id = 0;
/**
 * Abstract base implementing a canvas overlay to be placed on the map.
 *
 * @export
 * @abstract
 * @class MapLabel
 */
var CanvasOverlay = /** @class */ (function () {
    /**
     * Creates a new instance of the CanvasOverlay class.
     * @param  {HTMLCanvasElements => void} drawCallback A callback function that is triggered when the canvas is ready to be
     * rendered for the current map view.
     * @memberof CanvasOverlay
     * @constructor
     */
    function CanvasOverlay(drawCallback) {
        var _this = this;
        this._canvasReady = new Promise(function (resolve, reject) { _this._readyResolver = resolve; });
        this._drawCallback = drawCallback;
        id++;
    }
    Object.defineProperty(CanvasOverlay.prototype, "CanvasReady", {
        /**
         * Returns a promise that gets resolved when the canvas overlay is ready for interaction.
         *
         * @readonly
         * @type {Promise<boolean>}
         * @memberof CanvasOverlay
         */
        get: function () { return this._canvasReady; },
        enumerable: true,
        configurable: true
    });
    ///
    /// Public methods
    ///
    /**
     * Deletes the canvas overlay.
     *
     * @memberof CanvasOverlay
     */
    CanvasOverlay.prototype.Delete = function () {
        this.SetMap(null);
    };
    /**
     * CanvasOverlay added to map, load canvas.
     * @memberof CanvasOverlay
     * @method
     * @public
     * @memberof CanvasOverlay
     */
    CanvasOverlay.prototype.OnAdd = function () {
        this._canvas = document.createElement('canvas');
        this._canvas.style.position = 'absolute';
        this._canvas.style.left = '0px';
        this._canvas.style.top = '0px';
        this._canvas.id = "xMapOverlay" + id;
        // Add the canvas to the overlay.
        this.SetCanvasElement(this._canvas);
    };
    ;
    /**
     * When the CanvasLayer is removed from the map, release resources.
     * @memberof CanvasOverlay
     * @method
     * @public
     */
    CanvasOverlay.prototype.OnRemove = function () {
        this.SetCanvasElement(null);
        this.RemoveEventHandlers();
        this._canvas = null;
    };
    /**
     * Redraws the canvas for the current map view.
     * @param {boolean} clear - True to clear the canvas before drawing.
     * @memberof CanvasOverlay
     * @method
     * @public
     */
    CanvasOverlay.prototype.Redraw = function (clear) {
        if (this._canvas == null) {
            return;
        }
        // Clear canvas by updating dimensions. This also ensures canvas stays the same size as the map.
        if (clear) {
            this.Resize();
        }
        // Call the drawing callback function if specified.
        if (this._drawCallback) {
            this._drawCallback(this._canvas);
        }
    };
    /**
     * Simple function for updating the CSS position and dimensions of the canvas.
     * @param x The horizontal offset position of the canvas.
     * @param y The vertical offset position of the canvas.
     * @param w The width of the canvas.
     * @param h The height of the canvas.
     * @memberof CanvasOverlay
     * @method
     * @protected
     */
    CanvasOverlay.prototype.UpdatePosition = function (x, y, w, h) {
        // Update CSS position.
        this._canvas.style.left = x + 'px';
        this._canvas.style.top = y + 'px';
        // Update CSS dimensions.
        this._canvas.style.width = w + 'px';
        this._canvas.style.height = h + 'px';
    };
    return CanvasOverlay;
}());
exports.CanvasOverlay = CanvasOverlay;
//# sourceMappingURL=canvas-overlay.js.map