"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Abstract class defining the contract for a polyline in the architecture specific implementation.
 *
 * @export
 * @abstract
 * @class Polyline
 */
var Polyline = /** @class */ (function () {
    function Polyline() {
    }
    Object.defineProperty(Polyline.prototype, "Center", {
        ///
        /// Property definitions
        ///
        /**
         * Gets the polyline's center.
         * @readonly
         * @public
         * @type {ILatLong}
         * @memberof Polyline
         */
        get: function () {
            if (this._center == null) {
                this._center = this.GetBoundingCenter();
            }
            return this._center;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Polyline.prototype, "Centroid", {
        /**
         * Gets the polyline's centroid.
         * @readonly
         * @public
         * @type {ILatLong}
         * @memberof Polyline
         */
        get: function () {
            if (this._centroid == null) {
                this._centroid = this.GetPolylineCentroid();
            }
            return this._centroid;
        },
        enumerable: true,
        configurable: true
    });
    ///
    /// Public methods
    ///
    /**
     * Get the centroid of the polyline based on the a path.
     *
     * @param {Array<ILatLong>} path - the path for which to generate the centroid
     * @return {ILatLong} - The centroid coordinates of the polyline.
     * @memberof Polyline
     * @method
     * @public
     * @static
     */
    Polyline.GetPolylineCentroid = function (path) {
        var c = { latitude: 0, longitude: 0 };
        var off = path[0];
        if (off != null) {
            var twicearea = 0;
            var x = 0;
            var y = 0;
            var p1 = void 0, p2 = void 0;
            var f = void 0;
            for (var i = 0, j = path.length - 1; i < path.length; j = i++) {
                p1 = path[i];
                p2 = path[j];
                f = (p1.latitude - off.latitude) * (p2.longitude - off.longitude) -
                    (p2.latitude - off.latitude) * (p1.longitude - off.longitude);
                twicearea += f;
                x += (p1.latitude + p2.latitude - 2 * off.latitude) * f;
                y += (p1.longitude + p2.longitude - 2 * off.longitude) * f;
            }
            if (twicearea !== 0) {
                f = twicearea * 3;
                c.latitude = x / f + off.latitude;
                c.longitude = y / f + off.longitude;
            }
            else {
                c.latitude = off.latitude;
                c.longitude = off.longitude;
            }
        }
        else {
            c = null;
        }
        return c;
    };
    ///
    /// Protected methods
    ///
    /**
     * Gets the center of the polyline' bounding box.
     *
     * @returns {ILatLong} - {@link ILatLong} object containing the center of the bounding box.
     * @memberof Polyline
     * @method
     * @protected
     */
    Polyline.prototype.GetBoundingCenter = function () {
        var c = { latitude: 0, longitude: 0 };
        var x1 = 90, x2 = -90, y1 = 180, y2 = -180;
        var path = this.GetPath();
        if (path) {
            path.forEach(function (p) {
                if (p.latitude < x1) {
                    x1 = p.latitude;
                }
                if (p.latitude > x2) {
                    x2 = p.latitude;
                }
                if (p.longitude < y1) {
                    y1 = p.longitude;
                }
                if (p.longitude > y2) {
                    y2 = p.longitude;
                }
            });
            c.latitude = x1 + (x2 - x1) / 2;
            c.longitude = y1 + (y2 - y1) / 2;
        }
        else {
            c = null;
        }
        return c;
    };
    /**
     * Get the centroid of the polyline based on the polyline path.
     *
     * @return {ILatLong} - The centroid coordinates of the polyline.
     * @memberof Polyline
     * @method
     * @protected
     */
    Polyline.prototype.GetPolylineCentroid = function () {
        var path = this.GetPath();
        var c = Polyline.GetPolylineCentroid(path);
        return c;
    };
    return Polyline;
}());
exports.Polyline = Polyline;
//# sourceMappingURL=polyline.js.map