"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var marker_1 = require("../../models/marker");
var google_conversions_1 = require("./google-conversions");
var google_marker_1 = require("../../models/google/google-marker");
/**
 * This abstract partially implements the contract for the {@link LayerService}
 * and {@link ClusterService} for the Google Maps archtiecture. It serves
 * as the base class for basic layer ({@link GoogleLayerService}) and cluster layer ({@link GoogleClusterLayer}).
 *
 * @export
 * @abstract
 * @class GoogleLayerBase
 */
var GoogleLayerBase = /** @class */ (function () {
    ///
    /// Constructor
    ///
    /**
     * Creates an instance of GoogleLayerBase.
     * @param {MapService} _mapService - Concrete {@link MapService} implementation for Google Maps.
     * An instance of {@link GoogleMapService}.
     *
     * @memberof GoogleLayerBase
     */
    function GoogleLayerBase(_mapService) {
        this._mapService = _mapService;
    }
    /**
     * Creates a marker in the layer.
     *
     * @param {number} layer - The Id of the layer in which to create the marker.
     * @param {IMarkerOptions} options - {@link IMarkerOptions} object containing the marker properties.
     * @returns {Promise<Marker>} - A promise that when fullfilled contains the {@link Marker} model for the created marker.
     *
     * @memberof GoogleLayerBase
     */
    GoogleLayerBase.prototype.CreateMarker = function (layer, options) {
        var mp = this._mapService.MapPromise;
        var lp = this._layers.get(layer);
        return Promise.all([mp, lp]).then(function (_a) {
            var map = _a[0], l = _a[1];
            var payload = function (x) {
                var marker = new google.maps.Marker(x);
                if (options.metadata) {
                    options.metadata.forEach(function (val, key) { return marker.Metadata.set(key, val); });
                }
                marker.setMap(map);
                var m = new google_marker_1.GoogleMarker(marker);
                m.IsFirst = options.isFirst;
                m.IsLast = options.isLast;
                if (options.metadata) {
                    options.metadata.forEach(function (val, key) { return m.Metadata.set(key, val); });
                }
                l.AddEntity(m);
                return m;
            };
            var o = google_conversions_1.GoogleConversions.TranslateMarkerOptions(options);
            if (options.iconInfo && options.iconInfo.markerType) {
                var s = marker_1.Marker.CreateMarker(options.iconInfo);
                if (typeof (s) === 'string') {
                    o.icon = s;
                    return payload(o);
                }
                else {
                    return s.then(function (x) {
                        o.icon = x.icon;
                        return payload(o);
                    });
                }
            }
            else {
                return payload(o);
            }
        });
    };
    ;
    /**
     * Creates an array of unbound markers. Use this method to create arrays of markers to be used in bulk
     * operations.
     *
     * @param {Array<IMarkerOptions>} options - Marker options defining the markers.
     * @param {IMarkerIconInfo} markerIcon - Optional information to generate custom markers. This will be applied to all markers.
     * @returns {Promise<Array<Marker>>} - A promise that when fullfilled contains the an arrays of the Marker models.
     *
     * @memberof GoogleLayerBase
     */
    GoogleLayerBase.prototype.CreateMarkers = function (options, markerIcon) {
        var payload = function (icon) {
            var markers = options.map(function (mo) {
                var o = google_conversions_1.GoogleConversions.TranslateMarkerOptions(mo);
                if (icon && icon !== '') {
                    o.icon = icon;
                }
                var pushpin = new google.maps.Marker(o);
                var marker = new google_marker_1.GoogleMarker(pushpin);
                marker.IsFirst = mo.isFirst;
                marker.IsLast = mo.isLast;
                if (mo.metadata) {
                    mo.metadata.forEach(function (val, key) { return marker.Metadata.set(key, val); });
                }
                return marker;
            });
            return markers;
        };
        var p = new Promise(function (resolve, reject) {
            if (markerIcon && markerIcon.markerType) {
                var s = marker_1.Marker.CreateMarker(markerIcon);
                if (typeof (s) === 'string') {
                    resolve(payload(s));
                }
                else {
                    return s.then(function (x) {
                        resolve(payload(x.icon));
                    });
                }
            }
            else {
                resolve(payload(null));
            }
        });
        return p;
    };
    ///
    /// Protected methods
    ///
    /**
     * Gets the layer based on its id.
     *
     * @protected
     * @param {number} id - Layer Id.
     * @returns {Promise<Layer>}  - A promise that when fullfilled contains the {@link Layer} model for the layer.
     *
     * @memberof GoogleLayerBase
     */
    GoogleLayerBase.prototype.GetLayerById = function (id) {
        var p;
        this._layers.forEach(function (l, k) { if (k === id) {
            p = l;
        } });
        return p;
    };
    return GoogleLayerBase;
}());
exports.GoogleLayerBase = GoogleLayerBase;
//# sourceMappingURL=google-layer-base.js.map