import { NgZone } from '@angular/core';
import { MapServiceFactory } from '../mapservicefactory';
import { MapService } from '../map.service';
import { MapAPILoader } from '../mapapiloader';
import { MarkerService } from '../marker.service';
import { LayerService } from '../layer.service';
import { ClusterService } from '../cluster.service';
import { PolygonService } from '../polygon.service';
import { PolylineService } from '../polyline.service';
import { GoogleInfoBoxService } from './google-infobox.service';
import { GoogleMarkerService } from './google-marker.service';
import { GoogleLayerService } from './google-layer.service';
import { GoogleClusterService } from './google-cluster.service';
/**
 * Implements a factory to create three necessary Google Maps specific service instances.
 *
 * @export
 * @class GoogleMapServiceFactory
 * @implements {MapServiceFactory}
 */
export declare class GoogleMapServiceFactory implements MapServiceFactory {
    private _loader;
    private _zone;
    private _map;
    private _mapResolver;
    /**
     * Creates an instance of GoogleMapServiceFactory.
     * @param {MapAPILoader} _loader - {@link MapAPILoader} implementation for the Google Map provider.
     * @param {NgZone} _zone - NgZone object to implement zone aware promises.
     *
     * @memberof GoogleMapServiceFactory
     */
    constructor(_loader: MapAPILoader, _zone: NgZone);
    /**
     * Creates the map service for the Google Maps implementation.
     *
     * @returns {MapService} - {@link MapService}. A concreted instance of the {@link GoogleMapService}.
     *
     * @memberof GoogleMapServiceFactory
     */
    Create(): MapService;
    /**
     * Creates the cluster service for the Google Maps implementation.
     *
     * @param {MapService} map - {@link MapService}. A concreted instance of the {@link GoogleMapService}.
     * @returns {ClusterService} - {@link ClusterService}. A concreted instance of the {@link GoogleClusterService}.
     *
     * @memberof GoogleMapServiceFactory
     */
    CreateClusterService(_mapService: MapService): ClusterService;
    /**
     * Creates thh info box service for the Google Maps implementation.
     *
     * @param {MapService} map - {@link MapService}. A concreted instance of the {@link GoogleMapService}.
     * @param {MarkerService} map - {@link MarkerService}. A concreted instance of the {@link GoogleMarkerService}.
     * @returns {InfoBoxService} - {@link InfoBoxService}. A concreted instance of the {@link GoogleInfoBoxService}.
     *
     * @memberof GoogleMapServiceFactory
     */
    CreateInfoBoxService(_mapService: MapService, _markerService: MarkerService): GoogleInfoBoxService;
    /**
     * Creates the layer service for the Google Maps implementation.
     *
     * @param {MapService} map - {@link MapService}. A concreted instance of the {@link GoogleMapService}.
     * @returns {LayerService} - {@link LayerService}. A concreted instance of the {@link GoogleLayerService}.
     *
     * @memberof GoogleMapServiceFactory
     */
    CreateLayerService(_mapService: MapService): GoogleLayerService;
    /**
     * Creates the marker service for the Google Maps implementation.
     *
     * @param {MapService} map - {@link MapService}. A concreted instance of the {@link GoogleMapService}.
     * @param {LayerService} layers - {@link LayerService}. A concreted instance of the {@link GoogleLayerService}.
     * @param {ClusterService} clusters  - {@link ClusterService}. A concreted instance of the {@link GoogleClusterService}.
     * @returns {MarkerService} - {@link MarkerService}. A concreted instance of the {@link GoogleMarkerService}.
     *
     * @memberof GoogleMapServiceFactory
     */
    CreateMarkerService(_mapService: MapService, _layerService: GoogleLayerService, _clusterService: GoogleClusterService): GoogleMarkerService;
    /**
     * Creates the polygon service for the Google Maps implementation.
     *
     * @param {MapService} map - {@link MapService} implementation for thh underlying map archticture.
     * @param {LayerService} layers - {@link LayerService} implementation for the underlying map architecture.
     * @returns {PolygonService} - {@link PolygonService} implementation for the underlying map architecture.
     *
     * @memberof MapServiceFactory
     */
    CreatePolygonService(map: MapService, layers: LayerService): PolygonService;
    /**
     * Creates the polyline service for the Google Maps implementation.
     *
     * @param {MapService} map - {@link MapService} implementation for thh underlying map archticture.
     * @param {LayerService} layers - {@link LayerService} implementation for the underlying map architecture.
     * @returns {PolylineService} - {@link PolylineService} implementation for the underlying map architecture.
     *
     * @memberof MapServiceFactory
     */
    CreatePolylineService(map: MapService, layers: LayerService): PolylineService;
}
/**
 *  Creates a new instance of a plaform specific MapServiceFactory.
 *
 * @param apiLoader - An {@link MapAPILoader} instance. This is expected to the a {@link GoogleMapAPILoader}.
 * @param zone - An NgZone instance to provide zone aware promises.
 *
 * @return {MapServiceFactory} -  A {@link MapServiceFactory} instance.
 */
export declare function GoogleMapServiceFactoryFactory(apiLoader: MapAPILoader, zone: NgZone): MapServiceFactory;
/**
 * Creates a new instance of a plaform specific MapLoaderFactory.
 *
 * @export
 * @returns {MapAPILoader} - A {@link MapAPILoader} instance.
 */
export declare function GoogleMapLoaderFactory(): MapAPILoader;
